import { ReactNode, ReactElement, AnchorHTMLAttributes, Ref, FC } from 'react';
import { Store, Event } from 'effector';
import { Location, State } from 'history';

declare type PathParams<PathParamKeys extends string = never> = [
    PathParamKeys
] extends [never] ? {} : Record<PathParamKeys, string>;
declare type Match<PathParamKeys extends string = never> = {
    params: PathParams<PathParamKeys>;
    isExact: boolean;
};
declare type HistoryMethod = "push" | "replace";
declare type NavigateOptions<PathParamKeys extends string = never> = Partial<{
    params: Partial<PathParams<PathParamKeys>>;
    search: boolean | string;
    hash: boolean | string;
    method: HistoryMethod;
}>;

declare type RouteChildrenFunction<PathParamKeys extends string = never> = (match: Match<PathParamKeys>) => ReactNode;
declare type RouteProps<PathParamKeys extends string = never> = {
    exact?: boolean;
    children?: ReactNode | RouteChildrenFunction<PathParamKeys>;
};
declare type Route<PathParamKeys extends string = never> = {
    (props: RouteProps<PathParamKeys>): ReactElement | null;
    path: string;
    match: Store<Match<PathParamKeys> | null>;
    open: Event<Match<PathParamKeys>>;
    close: Event<null>;
    status: Store<boolean>;
    navigate: Event<void | NavigateOptions<PathParamKeys>>;
};
declare const createRoute: <PathParamKeys extends string = never>(path: string) => Route<PathParamKeys>;
declare type CurrentRoute = {
    path: null;
    navigate: Event<NavigateOptions<never>>;
};
declare const CurrentRoute: CurrentRoute;

declare const $location: Store<Location<State>>;
declare const $pathname: Store<string>;
declare const $search: Store<string>;
declare const $hash: Store<string>;

interface LinkProps<PathParamKeys extends string = never> extends Omit<AnchorHTMLAttributes<HTMLAnchorElement>, "href">, NavigateOptions<PathParamKeys> {
    to: Route<PathParamKeys> | CurrentRoute;
}
declare const Link: <PathParamKeys extends string = never>(props: LinkProps<PathParamKeys> & {
    ref?: Ref<HTMLAnchorElement> | undefined;
}) => ReactElement;

interface RedirectProps<PathParamKeys extends string = never> extends NavigateOptions<PathParamKeys> {
    to: Route<PathParamKeys> | CurrentRoute;
}
declare const Redirect: <PathParamKeys extends string = never>({ to: Route, method, ...options }: RedirectProps<PathParamKeys>) => null;

declare const Switch: FC;

export { $hash, $location, $pathname, $search, CurrentRoute, HistoryMethod, Link, LinkProps, Match, NavigateOptions, PathParams, Redirect, RedirectProps, Route, RouteProps, Switch, createRoute };
